@@ -4,6 +4,7 @@ use strict;
 use utf8;
 use FindBin '$Bin';
 use JSON::Create 'create_json';
+binmode STDOUT, ":encoding(utf8)";
 my %crazyhash = (
     'code' => sub { return "강남스타일"; },
     'regex' => qr/.*/,
@@ -12,6 +13,7 @@ my %crazyhash = (
 my $jc = JSON::Create->new ();
 # Let's validate the output of the subroutine below.
 $jc->validate (1);
+$jc->indent (1);
 # Try this one weird old trick to convert your Perl type.
 $jc->type_handler (
     sub {
@@ -27,4 +29,4 @@ $jc->type_handler (
 	return create_json ({ type => $type, value => $value, });
     }
 );
-print $jc->run (\%crazyhash);
+print $jc->run (\%crazyhash), "\n";
